@@ -23,7 +23,7 @@ import (
 
 	"github.com/google/go-cmp/cmp"
 	v1 "k8s.io/api/core/v1"
-	networkingv1alpha1 "k8s.io/api/networking/v1alpha1"
+	networkingv1 "k8s.io/api/networking/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/util/wait"
@@ -54,25 +54,25 @@ func newFakeRepair() (*fake.Clientset, *fakeRepair) {
 	serviceInformer := informerFactory.Core().V1().Services()
 	serviceIndexer := serviceInformer.Informer().GetIndexer()
 
-	serviceCIDRInformer := informerFactory.Networking().V1alpha1().ServiceCIDRs()
+	serviceCIDRInformer := informerFactory.Networking().V1().ServiceCIDRs()
 	serviceCIDRIndexer := serviceCIDRInformer.Informer().GetIndexer()
 
-	ipInformer := informerFactory.Networking().V1alpha1().IPAddresses()
+	ipInformer := informerFactory.Networking().V1().IPAddresses()
 	ipIndexer := ipInformer.Informer().GetIndexer()
 
 	fakeClient.PrependReactor("create", "ipaddresses", k8stesting.ReactionFunc(func(action k8stesting.Action) (bool, runtime.Object, error) {
-		ip := action.(k8stesting.CreateAction).GetObject().(*networkingv1alpha1.IPAddress)
+		ip := action.(k8stesting.CreateAction).GetObject().(*networkingv1.IPAddress)
 		err := ipIndexer.Add(ip)
 		return false, ip, err
 	}))
 	fakeClient.PrependReactor("update", "ipaddresses", k8stesting.ReactionFunc(func(action k8stesting.Action) (bool, runtime.Object, error) {
-		ip := action.(k8stesting.UpdateAction).GetObject().(*networkingv1alpha1.IPAddress)
+		ip := action.(k8stesting.UpdateAction).GetObject().(*networkingv1.IPAddress)
 		return false, ip, fmt.Errorf("IPAddress is inmutable after creation")
 	}))
 	fakeClient.PrependReactor("delete", "ipaddresses", k8stesting.ReactionFunc(func(action k8stesting.Action) (bool, runtime.Object, error) {
 		ip := action.(k8stesting.DeleteAction).GetName()
 		err := ipIndexer.Delete(ip)
-		return false, &networkingv1alpha1.IPAddress{}, err
+		return false, &networkingv1.IPAddress{}, err
 	}))
 
 	r := NewRepairIPAddress(0*time.Second,
@@ -88,8 +88,8 @@ func TestRepairServiceIP(t *testing.T) {
 	tests := []struct {
 		name        string
 		svcs        []*v1.Service
-		ipAddresses []*networkingv1alpha1.IPAddress
-		cidrs       []*networkingv1alpha1.ServiceCIDR
+		ipAddresses []*networkingv1.IPAddress
+		cidrs       []*networkingv1.ServiceCIDR
 		expectedIPs []string
 		actions     [][]string // verb and resource
 		events      []string
@@ -97,10 +97,10 @@ func TestRepairServiceIP(t *testing.T) {
 		{
 			name: "no changes needed single stack",
 			svcs: []*v1.Service{newService("test-svc", []string{"10.0.1.1"})},
-			ipAddresses: []*networkingv1alpha1.IPAddress{
+			ipAddresses: []*networkingv1.IPAddress{
 				newIPAddress("10.0.1.1", newService("test-svc", []string{"10.0.1.1"})),
 			},
-			cidrs: []*networkingv1alpha1.ServiceCIDR{
+			cidrs: []*networkingv1.ServiceCIDR{
 				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
 			},
 			expectedIPs: []string{"10.0.1.1"},
@@ -110,11 +110,11 @@ func TestRepairServiceIP(t *testing.T) {
 		{
 			name: "no changes needed dual stack",
 			svcs: []*v1.Service{newService("test-svc", []string{"10.0.1.1", "2001:db8::10"})},
-			ipAddresses: []*networkingv1alpha1.IPAddress{
+			ipAddresses: []*networkingv1.IPAddress{
 				newIPAddress("10.0.1.1", newService("test-svc", []string{"10.0.1.1"})),
 				newIPAddress("2001:db8::10", newService("test-svc", []string{"2001:db8::10"})),
 			},
-			cidrs: []*networkingv1alpha1.ServiceCIDR{
+			cidrs: []*networkingv1.ServiceCIDR{
 				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
 			},
 			expectedIPs: []string{"10.0.1.1", "2001:db8::10"},
@@ -124,11 +124,11 @@ func TestRepairServiceIP(t *testing.T) {
 		{
 			name: "no changes needed dual stack multiple cidrs",
 			svcs: []*v1.Service{newService("test-svc", []string{"192.168.0.1", "2001:db8:a:b::10"})},
-			ipAddresses: []*networkingv1alpha1.IPAddress{
+			ipAddresses: []*networkingv1.IPAddress{
 				newIPAddress("192.168.0.1", newService("test-svc", []string{"192.168.0.1"})),
 				newIPAddress("2001:db8:a:b::10", newService("test-svc", []string{"2001:db8:a:b::10"})),
 			},
-			cidrs: []*networkingv1alpha1.ServiceCIDR{
+			cidrs: []*networkingv1.ServiceCIDR{
 				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
 				newServiceCIDR("custom", "192.168.0.0/24", "2001:db8:a:b::/64"),
 			},
@@ -140,7 +140,7 @@ func TestRepairServiceIP(t *testing.T) {
 		{
 			name: "create IPAddress single stack",
 			svcs: []*v1.Service{newService("test-svc", []string{"10.0.1.1"})},
-			cidrs: []*networkingv1alpha1.ServiceCIDR{
+			cidrs: []*networkingv1.ServiceCIDR{
 				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
 			},
 			expectedIPs: []string{"10.0.1.1"},
@@ -150,7 +150,7 @@ func TestRepairServiceIP(t *testing.T) {
 		{
 			name: "create IPAddresses dual stack",
 			svcs: []*v1.Service{newService("test-svc", []string{"10.0.1.1", "2001:db8::10"})},
-			cidrs: []*networkingv1alpha1.ServiceCIDR{
+			cidrs: []*networkingv1.ServiceCIDR{
 				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
 			},
 			expectedIPs: []string{"10.0.1.1", "2001:db8::10"},
@@ -163,7 +163,7 @@ func TestRepairServiceIP(t *testing.T) {
 		{
 			name: "create IPAddress single stack from secondary",
 			svcs: []*v1.Service{newService("test-svc", []string{"192.168.1.1"})},
-			cidrs: []*networkingv1alpha1.ServiceCIDR{
+			cidrs: []*networkingv1.ServiceCIDR{
 				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
 				newServiceCIDR("custom", "192.168.1.0/24", ""),
 			},
@@ -174,10 +174,10 @@ func TestRepairServiceIP(t *testing.T) {
 		{
 			name: "reconcile IPAddress single stack wrong reference",
 			svcs: []*v1.Service{newService("test-svc", []string{"10.0.1.1"})},
-			ipAddresses: []*networkingv1alpha1.IPAddress{
+			ipAddresses: []*networkingv1.IPAddress{
 				newIPAddress("10.0.1.1", newService("test-svc2", []string{"10.0.1.1"})),
 			},
-			cidrs: []*networkingv1alpha1.ServiceCIDR{
+			cidrs: []*networkingv1.ServiceCIDR{
 				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
 			},
 			expectedIPs: []string{"10.0.1.1"},
@@ -187,11 +187,11 @@ func TestRepairServiceIP(t *testing.T) {
 		{
 			name: "reconcile IPAddresses dual stack",
 			svcs: []*v1.Service{newService("test-svc", []string{"10.0.1.1", "2001:db8::10"})},
-			ipAddresses: []*networkingv1alpha1.IPAddress{
+			ipAddresses: []*networkingv1.IPAddress{
 				newIPAddress("10.0.1.1", newService("test-svc2", []string{"10.0.1.1"})),
 				newIPAddress("2001:db8::10", newService("test-svc2", []string{"2001:db8::10"})),
 			},
-			cidrs: []*networkingv1alpha1.ServiceCIDR{
+			cidrs: []*networkingv1.ServiceCIDR{
 				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
 			},
 			expectedIPs: []string{"10.0.1.1", "2001:db8::10"},
@@ -204,11 +204,11 @@ func TestRepairServiceIP(t *testing.T) {
 		{
 			name: "one IP out of range",
 			svcs: []*v1.Service{newService("test-svc", []string{"192.168.1.1", "2001:db8::10"})},
-			ipAddresses: []*networkingv1alpha1.IPAddress{
+			ipAddresses: []*networkingv1.IPAddress{
 				newIPAddress("192.168.1.1", newService("test-svc", []string{"192.168.1.1"})),
 				newIPAddress("2001:db8::10", newService("test-svc", []string{"2001:db8::10"})),
 			},
-			cidrs: []*networkingv1alpha1.ServiceCIDR{
+			cidrs: []*networkingv1.ServiceCIDR{
 				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
 			},
 			expectedIPs: []string{"2001:db8::10"},
@@ -217,10 +217,10 @@ func TestRepairServiceIP(t *testing.T) {
 		},
 		{
 			name: "one IP orphan",
-			ipAddresses: []*networkingv1alpha1.IPAddress{
+			ipAddresses: []*networkingv1.IPAddress{
 				newIPAddress("10.0.1.1", newService("test-svc", []string{"10.0.1.1"})),
 			},
-			cidrs: []*networkingv1alpha1.ServiceCIDR{
+			cidrs: []*networkingv1.ServiceCIDR{
 				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
 			},
 			actions: [][]string{{"delete", "ipaddresses"}},
@@ -229,10 +229,10 @@ func TestRepairServiceIP(t *testing.T) {
 		{
 			name: "one IP out of range matching the network address",
 			svcs: []*v1.Service{newService("test-svc", []string{"10.0.0.0"})},
-			ipAddresses: []*networkingv1alpha1.IPAddress{
+			ipAddresses: []*networkingv1.IPAddress{
 				newIPAddress("10.0.0.0", newService("test-svc", []string{"10.0.0.0"})),
 			},
-			cidrs: []*networkingv1alpha1.ServiceCIDR{
+			cidrs: []*networkingv1.ServiceCIDR{
 				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
 			},
 			expectedIPs: []string{"10.0.0.0"},
@@ -242,10 +242,10 @@ func TestRepairServiceIP(t *testing.T) {
 		{
 			name: "one IP out of range matching the broadcast address",
 			svcs: []*v1.Service{newService("test-svc", []string{"10.0.255.255"})},
-			ipAddresses: []*networkingv1alpha1.IPAddress{
+			ipAddresses: []*networkingv1.IPAddress{
 				newIPAddress("10.0.255.255", newService("test-svc", []string{"10.0.255.255"})),
 			},
-			cidrs: []*networkingv1alpha1.ServiceCIDR{
+			cidrs: []*networkingv1.ServiceCIDR{
 				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
 			},
 			expectedIPs: []string{"10.0.255.255"},
@@ -255,10 +255,10 @@ func TestRepairServiceIP(t *testing.T) {
 		{
 			name: "one IPv6 out of range matching the subnet address",
 			svcs: []*v1.Service{newService("test-svc", []string{"2001:db8::"})},
-			ipAddresses: []*networkingv1alpha1.IPAddress{
+			ipAddresses: []*networkingv1.IPAddress{
 				newIPAddress("2001:db8::", newService("test-svc", []string{"2001:db8::"})),
 			},
-			cidrs: []*networkingv1alpha1.ServiceCIDR{
+			cidrs: []*networkingv1.ServiceCIDR{
 				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
 			},
 			expectedIPs: []string{"2001:db8::"},
@@ -268,20 +268,20 @@ func TestRepairServiceIP(t *testing.T) {
 		{
 			name: "one IPv6 matching the broadcast address",
 			svcs: []*v1.Service{newService("test-svc", []string{"2001:db8::ffff:ffff:ffff:ffff"})},
-			ipAddresses: []*networkingv1alpha1.IPAddress{
+			ipAddresses: []*networkingv1.IPAddress{
 				newIPAddress("2001:db8::ffff:ffff:ffff:ffff", newService("test-svc", []string{"2001:db8::ffff:ffff:ffff:ffff"})),
 			},
-			cidrs: []*networkingv1alpha1.ServiceCIDR{
+			cidrs: []*networkingv1.ServiceCIDR{
 				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
 			},
 			expectedIPs: []string{"2001:db8::ffff:ffff:ffff:ffff"},
 		},
 		{
 			name: "one IP orphan matching the broadcast address",
-			ipAddresses: []*networkingv1alpha1.IPAddress{
+			ipAddresses: []*networkingv1.IPAddress{
 				newIPAddress("10.0.255.255", newService("test-svc", []string{"10.0.255.255"})),
 			},
-			cidrs: []*networkingv1alpha1.ServiceCIDR{
+			cidrs: []*networkingv1.ServiceCIDR{
 				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
 			},
 			actions: [][]string{{"delete", "ipaddresses"}},
@@ -290,11 +290,11 @@ func TestRepairServiceIP(t *testing.T) {
 		{
 			name: "Two IPAddresses referencing the same service",
 			svcs: []*v1.Service{newService("test-svc", []string{"10.0.1.1"})},
-			ipAddresses: []*networkingv1alpha1.IPAddress{
+			ipAddresses: []*networkingv1.IPAddress{
 				newIPAddress("10.0.1.1", newService("test-svc", []string{"10.0.1.1"})),
 				newIPAddress("10.0.1.2", newService("test-svc", []string{"10.0.1.1"})),
 			},
-			cidrs: []*networkingv1alpha1.ServiceCIDR{
+			cidrs: []*networkingv1.ServiceCIDR{
 				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
 			},
 			actions: [][]string{{"delete", "ipaddresses"}},
@@ -306,10 +306,10 @@ func TestRepairServiceIP(t *testing.T) {
 				newService("test-svc", []string{"10.0.1.1"}),
 				newService("test-svc2", []string{"10.0.1.1"}),
 			},
-			ipAddresses: []*networkingv1alpha1.IPAddress{
+			ipAddresses: []*networkingv1.IPAddress{
 				newIPAddress("10.0.1.1", newService("test-svc2", []string{"10.0.1.1"})),
 			},
-			cidrs: []*networkingv1alpha1.ServiceCIDR{
+			cidrs: []*networkingv1.ServiceCIDR{
 				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
 			},
 			events: []string{"Warning ClusterIPAlreadyAllocated Cluster IP [IPv4]:10.0.1.1 was assigned to multiple services; please recreate service"},
@@ -327,10 +327,7 @@ func TestRepairServiceIP(t *testing.T) {
 					t.Errorf("Unexpected error trying to add Service %v object: %v", cidr, err)
 				}
 			}
-			err := r.syncCIDRs()
-			if err != nil {
-				t.Fatal(err)
-			}
+
 			// override for testing
 			r.servicesSynced = func() bool { return true }
 			r.ipAddressSynced = func() bool { return true }
@@ -352,8 +349,7 @@ func TestRepairServiceIP(t *testing.T) {
 				}
 			}
 
-			err = r.runOnce()
-			if err != nil {
+			if err := r.runOnce(); err != nil {
 				t.Fatal(err)
 			}
 
@@ -374,23 +370,23 @@ func TestRepairServiceIP(t *testing.T) {
 func TestRepairIPAddress_syncIPAddress(t *testing.T) {
 	tests := []struct {
 		name    string
-		ip      *networkingv1alpha1.IPAddress
+		ip      *networkingv1.IPAddress
 		actions [][]string // verb and resource
 		wantErr bool
 	}{
 		{
 			name: "correct ipv4 address",
-			ip: &networkingv1alpha1.IPAddress{
+			ip: &networkingv1.IPAddress{
 				ObjectMeta: metav1.ObjectMeta{
 					Name: "10.0.1.1",
 					Labels: map[string]string{
-						networkingv1alpha1.LabelIPAddressFamily: string(v1.IPv4Protocol),
-						networkingv1alpha1.LabelManagedBy:       ipallocator.ControllerName,
+						networkingv1.LabelIPAddressFamily: string(v1.IPv4Protocol),
+						networkingv1.LabelManagedBy:       ipallocator.ControllerName,
 					},
 					CreationTimestamp: metav1.Date(2012, 1, 1, 0, 0, 0, 0, time.UTC),
 				},
-				Spec: networkingv1alpha1.IPAddressSpec{
-					ParentRef: &networkingv1alpha1.ParentReference{
+				Spec: networkingv1.IPAddressSpec{
+					ParentRef: &networkingv1.ParentReference{
 						Group:     "",
 						Resource:  "services",
 						Name:      "foo",
@@ -401,17 +397,17 @@ func TestRepairIPAddress_syncIPAddress(t *testing.T) {
 		},
 		{
 			name: "correct ipv6 address",
-			ip: &networkingv1alpha1.IPAddress{
+			ip: &networkingv1.IPAddress{
 				ObjectMeta: metav1.ObjectMeta{
 					Name: "2001:db8::11",
 					Labels: map[string]string{
-						networkingv1alpha1.LabelIPAddressFamily: string(v1.IPv6Protocol),
-						networkingv1alpha1.LabelManagedBy:       ipallocator.ControllerName,
+						networkingv1.LabelIPAddressFamily: string(v1.IPv6Protocol),
+						networkingv1.LabelManagedBy:       ipallocator.ControllerName,
 					},
 					CreationTimestamp: metav1.Date(2012, 1, 1, 0, 0, 0, 0, time.UTC),
 				},
-				Spec: networkingv1alpha1.IPAddressSpec{
-					ParentRef: &networkingv1alpha1.ParentReference{
+				Spec: networkingv1.IPAddressSpec{
+					ParentRef: &networkingv1.ParentReference{
 						Group:     "",
 						Resource:  "services",
 						Name:      "foo",
@@ -422,17 +418,17 @@ func TestRepairIPAddress_syncIPAddress(t *testing.T) {
 		},
 		{
 			name: "not managed by this controller",
-			ip: &networkingv1alpha1.IPAddress{
+			ip: &networkingv1.IPAddress{
 				ObjectMeta: metav1.ObjectMeta{
 					Name: "2001:db8::11",
 					Labels: map[string]string{
-						networkingv1alpha1.LabelIPAddressFamily: string(v1.IPv6Protocol),
-						networkingv1alpha1.LabelManagedBy:       "controller-foo",
+						networkingv1.LabelIPAddressFamily: string(v1.IPv6Protocol),
+						networkingv1.LabelManagedBy:       "controller-foo",
 					},
 					CreationTimestamp: metav1.Date(2012, 1, 1, 0, 0, 0, 0, time.UTC),
 				},
-				Spec: networkingv1alpha1.IPAddressSpec{
-					ParentRef: &networkingv1alpha1.ParentReference{
+				Spec: networkingv1.IPAddressSpec{
+					ParentRef: &networkingv1.ParentReference{
 						Group:     "networking.gateway.k8s.io",
 						Resource:  "gateway",
 						Name:      "foo",
@@ -443,17 +439,17 @@ func TestRepairIPAddress_syncIPAddress(t *testing.T) {
 		},
 		{
 			name: "out of range",
-			ip: &networkingv1alpha1.IPAddress{
+			ip: &networkingv1.IPAddress{
 				ObjectMeta: metav1.ObjectMeta{
 					Name: "fd00:db8::11",
 					Labels: map[string]string{
-						networkingv1alpha1.LabelIPAddressFamily: string(v1.IPv6Protocol),
-						networkingv1alpha1.LabelManagedBy:       ipallocator.ControllerName,
+						networkingv1.LabelIPAddressFamily: string(v1.IPv6Protocol),
+						networkingv1.LabelManagedBy:       ipallocator.ControllerName,
 					},
 					CreationTimestamp: metav1.Date(2012, 1, 1, 0, 0, 0, 0, time.UTC),
 				},
-				Spec: networkingv1alpha1.IPAddressSpec{
-					ParentRef: &networkingv1alpha1.ParentReference{
+				Spec: networkingv1.IPAddressSpec{
+					ParentRef: &networkingv1.ParentReference{
 						Group:     "",
 						Resource:  "services",
 						Name:      "foo",
@@ -464,17 +460,17 @@ func TestRepairIPAddress_syncIPAddress(t *testing.T) {
 		},
 		{
 			name: "leaked ip",
-			ip: &networkingv1alpha1.IPAddress{
+			ip: &networkingv1.IPAddress{
 				ObjectMeta: metav1.ObjectMeta{
 					Name: "10.0.1.1",
 					Labels: map[string]string{
-						networkingv1alpha1.LabelIPAddressFamily: string(v1.IPv6Protocol),
-						networkingv1alpha1.LabelManagedBy:       ipallocator.ControllerName,
+						networkingv1.LabelIPAddressFamily: string(v1.IPv6Protocol),
+						networkingv1.LabelManagedBy:       ipallocator.ControllerName,
 					},
 					CreationTimestamp: metav1.Date(2012, 1, 1, 0, 0, 0, 0, time.UTC),
 				},
-				Spec: networkingv1alpha1.IPAddressSpec{
-					ParentRef: &networkingv1alpha1.ParentReference{
+				Spec: networkingv1.IPAddressSpec{
+					ParentRef: &networkingv1.ParentReference{
 						Group:     "",
 						Resource:  "services",
 						Name:      "noexist",
@@ -526,12 +522,12 @@ func newService(name string, ips []string) *v1.Service {
 	return svc
 }
 
-func newServiceCIDR(name, primary, secondary string) *networkingv1alpha1.ServiceCIDR {
-	serviceCIDR := &networkingv1alpha1.ServiceCIDR{
+func newServiceCIDR(name, primary, secondary string) *networkingv1.ServiceCIDR {
+	serviceCIDR := &networkingv1.ServiceCIDR{
 		ObjectMeta: metav1.ObjectMeta{
 			Name: name,
 		},
-		Spec: networkingv1alpha1.ServiceCIDRSpec{},
+		Spec: networkingv1.ServiceCIDRSpec{},
 	}
 	serviceCIDR.Spec.CIDRs = append(serviceCIDR.Spec.CIDRs, primary)
 	if secondary != "" {
