@@ -16,4 +16,4 @@ limitations under the License.
 
 // Package resourcequota enforces all incoming requests against any applied quota
 // in the namespace context of the request
-package resourcequota // import "k8s.io/apiserver/pkg/admission/plugin/resourcequota"
+package resourcequota
